package signature.edgecolored;

public class EdgeColoredGraphFactory {
	
    public static EdgeColoredGraph makeHyperGraph() {
        EdgeColoredGraph g = new EdgeColoredGraph();
        
        return g;
    }

}
